clear all
close all
clc

warning off

vdata = xlsread('ORZ_AEA_Data.xls',2); 
time=vdata(:,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%IMPORTANT INPUT (sample window)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
t1=find(time==1921);
t2=find(time==2011.75);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%IMPORTANT INPUT 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
threshold=7;

vdata=vdata(t1:t2,:); 
q=vdata(:,1);
ngov=vdata(:,2);
ngdp=vdata(:,3);
pgdp=vdata(:,4);
totpop=vdata(:,5);
recession=vdata(:,6);
unemp=vdata(:,7);
pdvmil=vdata(:,8);
realgdp=vdata(:,9);

%Creating variables of interest

for t=2:size(pdvmil,1)
pdvmily(t,1)= pdvmil(t,1)./ngdp(t-1,1);
end

rgdp=(realgdp./totpop);
rgov=(ngov./totpop./pgdp);

lrgdp=log(realgdp./totpop);
lrgov=log(ngov./totpop./pgdp);


rrr=find(unemp>=threshold); 
rrrn=rrr+1; %Since unemployment above threshold in the previous period is the criterion
if rrrn(end)==length(unemp)+1
    rrrn=rrrn(1:end-1);
else
    rrrn=rrrn;
end
fu=zeros(size(unemp));
fu(rrrn)=1; % This is the dummy variable that defines the two states


nlags=5; %Number of lags we are interested in +1
y=lrgdp(nlags:end);
fu=fu(nlags:end);

Ly=lrgdp(nlags-1:end-1);
L2y=lrgdp(nlags-2:end-2);
L3y=lrgdp(nlags-3:end-3);
L4y=lrgdp(nlags-4:end-4);

Lg=lrgov(nlags-1:end-1);
L2g=lrgov(nlags-2:end-2);
L3g=lrgov(nlags-3:end-3);
L4g=lrgov(nlags-4:end-4);


constant= ones(length(y),1);
t=1:1:length(y);
tsq=t.^2;
tcu=t.^3;
tqu=t.^4;

%Creating part of the new variables that enter the RHS, which are in
%levels, not logs.
ynew=rgdp(nlags:end);
Lynew=rgdp(nlags-1:end-1);

gnew=rgov(nlags:end);
Lgnew=rgov(nlags-1:end-1);

ramey=pdvmily(nlags:end); % shock of interest

hor=20;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%LINEAR
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

x=[constant, t', tsq', tcu', tqu', ramey, Lg, L2g, L3g, L4g, Ly, L2y, L3y, L4y];
rpos=6; %position of the shock
[r,nn]=size(x);

for i=1:hor
    yy=(ynew(i:end)- Lynew(1:end-i+1))./Lynew(1:end-i+1); %Y variable entering in the LHS
    results=nwest(yy, x(1:end-i+1,:),i);
    b=results.beta;
    bint(:,1)=b-(results.se*1.96);
    bint(:,2)=b+(results.se*1.96);
    se(:,i)=results.se';
    reglin(:,i)=b;
    confidencelin(:,:,i)=bint;
end
confidencelin=reshape(confidencelin,2*nn,hor);
linGDP=[reglin(rpos,:);se(rpos,:)];

clear reg confidence bint se

y=lrgov(nlags:end);
for i=1:hor
    yy=(gnew(i:end)- Lgnew(1:end-i+1))./Lynew(1:end-i+1); %G variable entering in the LHS
    results=nwest(yy, x(1:end-i+1,:),i);
    b=results.beta;
    bint(:,1)=b-(results.se*1.96);
    bint(:,2)=b+(results.se*1.96);
    reg(:,i)=b;
    se(:,i)=results.se';
    confidence(:,:,i)=bint;
end
confidence=reshape(confidence,2*nn,hor);
linGOV=[reg(rpos,:);se(rpos,:)];

zz=zeros(1,hor);
figure(5)
subplot(1,2,1)
plot(1:1:hor, reg(rpos,:))
hold on
plot(1:1:hor, zz, 'k-')
hold on
grpyat=[(1:1:hor)', confidence(rpos,:)'; (hor:-1:1)' confidence(rpos+nn,hor:-1:1)'];
patch(grpyat(:,1), grpyat(:,2), [0.7 0.7 0.7],'edgecolor', [0.7 0.7 0.7]);
plot(1:1:hor, zz, 'k-')
plot(1:1:hor, reg(rpos,:), 'k','LineWidth', 1.5)
title('Government spending');
xlabel('quarter')
axis tight
subplot(1,2,2)
plot(1:1:hor, reglin(rpos,:))
hold on
plot(1:1:hor, zz, 'k-')
hold on
grpyat=[(1:1:hor)', confidencelin(rpos,:)'; (hor:-1:1)' confidencelin(rpos+nn,hor:-1:1)'];
patch(grpyat(:,1), grpyat(:,2), [0.7 0.7 0.7],'edgecolor', [0.7 0.7 0.7]);%[0.65 0.65 0.65]);
plot(1:1:hor, zz, 'k-')
plot(1:1:hor, reglin(rpos,:), 'k','LineWidth', 1.5)
title('GDP');
xlabel('quarter')
axis tight



% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %NON-LINEAR
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear reg b bint se
clear confidence

x=[constant, t', tsq', tcu', tqu', fu, fu.*ramey, (1-fu).*ramey, fu.*Lg, fu.*L2g, fu.*L3g, fu.*L4g, fu.*Ly, fu.*L2y, fu.*L3y, fu.*L4y, ...
                                                (1-fu).*Lg, (1-fu).*L2g, (1-fu).*L3g, (1-fu).*L4g, (1-fu).*Ly, (1-fu).*L2y, (1-fu).*L3y, (1-fu).*L4y];
[r,nn]=size(x);
rpos=7;%position of the shock

for i=1:hor
    yy=(ynew(i:end)- Lynew(1:end-i+1))./Lynew(1:end-i+1); %Y variable entering in the LHS
    results=nwest(yy, x(1:end-i+1,:),i);
    b=results.beta;
    bint(:,1)=b-(results.se*1.96);
    bint(:,2)=b+(results.se*1.96);
    se(:,i)=results.se';
    regy(:,i)=b;
    confidencey(:,:,i)=bint;
end
confidencey=reshape(confidencey,2*nn,hor);
stateaGDP=[regy(rpos,:);se(rpos,:)];
statebGDP=[regy(rpos+1,:);se(rpos+1,:)];


clear reg b bint se
clear confidence


x=[constant, t', tsq', tcu', tqu', fu, fu.*ramey, (1-fu).*ramey, fu.*Lg, fu.*L2g, fu.*L3g, fu.*L4g, fu.*Ly, fu.*L2y, fu.*L3y, fu.*L4y, ...
                                                (1-fu).*Lg, (1-fu).*L2g, (1-fu).*L3g, (1-fu).*L4g, (1-fu).*Ly, (1-fu).*L2y, (1-fu).*L3y, (1-fu).*L4y];
[r,nn]=size(x);


for i=1:hor
    yy=(gnew(i:end)- Lgnew(1:end-i+1))./Lynew(1:end-i+1); %G variable entering in the LHS
    results=nwest(yy, x(1:end-i+1,:),i);
    b=results.beta;
    bint(:,1)=b-(results.se*1.96);
    bint(:,2)=b+(results.se*1.96);
    se(:,i)=results.se';
    reg(:,i)=b;
    confidence(:,:,i)=bint;
end
confidence=reshape(confidence,2*nn,hor);
stateaGOV=[reg(rpos,:);se(rpos,:)];
statebGOV=[reg(rpos+1,:);se(rpos+1,:)];


figure(6)
subplot(1,2,1)
plot(1:1:hor, reg(rpos,:),'k')
hold on
grpyat=[(1:1:hor)', confidence(rpos,:)'; (hor:-1:1)' confidence(rpos+nn,hor:-1:1)'];
patch(grpyat(:,1), grpyat(:,2), [0.7 0.7 0.7],'edgecolor', [0.7 0.7 0.7]);
plot(1:1:hor, reg(rpos,:), 'k','LineWidth', 1.5)
hold on
plot(1:1:hor, reg(rpos+1,:), 'r-o', 1:1:hor, confidence(rpos+1,:), 'r--', 1:1:hor, confidence(rpos+1+nn,:), 'r--', 'LineWidth', 1.5);
axis tight
title('Government Spending')
xlabel('quarter')
subplot(1,2,2)
plot(1:1:hor, regy(rpos,:),'k')
hold on
grpyat=[(1:1:hor)', confidencey(rpos,:)'; (hor:-1:1)' confidencey(rpos+nn,hor:-1:1)'];
patch(grpyat(:,1), grpyat(:,2), [0.7 0.7 0.7],'edgecolor', [0.7 0.7 0.7]);
plot(1:1:hor, regy(rpos,:), 'k','LineWidth', 1.5)
hold on
plot(1:1:hor, regy(rpos+1,:), 'r-o', 1:1:hor, confidencey(rpos+1,:), 'r--', 1:1:hor, confidencey(rpos+1+nn,:), 'r--', 'LineWidth', 1.5);
axis tight
title('GDP')
xlabel('quarter')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Multipliers:

topasteinExcel=[linGDP;linGOV;stateaGDP;statebGDP;stateaGOV;statebGOV];
disp ('Output multipliers for Canada')
disp ( 'The multipliers by row are: 2 year integral multiplier, 4 year integral multiplier and peak multiplier')
disp('LINEAR ')
[sum(linGDP(1,1:8))./sum(linGOV(1,1:8)), sum(linGDP(1,1:16))./sum(linGOV(1,1:16)), max(linGDP(1,1:16))./max(linGOV(1,1:16))]
disp('HIGH UNEMPLOYMENT state')
[sum(stateaGDP(1,1:8))./sum(stateaGOV(1,1:8)), sum(stateaGDP(1,1:16))./sum(stateaGOV(1,1:16)), max(stateaGDP(1,1:16))./max(stateaGOV(1,1:16))]
disp('LOW UNEMPLOYMENT state')
[sum(statebGDP(1,1:8))./sum(statebGOV(1,1:8)), sum(statebGDP(1,1:16))./sum(statebGOV(1,1:16)), max(statebGDP(1,1:16))./max(statebGOV(1,1:16))]

